MODULE icealb
   !!======================================================================
   !!                       ***  MODULE  icealb  ***
   !! Ocean forcing:  bulk thermohaline forcing of the ice
   !!=====================================================================
   !! History :  4.0  ! 2017-07  (C. Rousset) Split ice and ocean albedos
   !!----------------------------------------------------------------------
#if defined key_lim3
   !!----------------------------------------------------------------------
   !!   'key_lim3'                                    LIM 3.0 sea-ice model
   !!----------------------------------------------------------------------
   !!   ice_alb       : albedo for ice (clear and overcast skies)
   !!   alb_init      : initialisation of albedo computation
   !!----------------------------------------------------------------------
   USE ice, ONLY: jpl ! sea-ice: number of categories
   USE phycst         ! physical constants
   !
   USE in_out_manager ! I/O manager
   USE lib_mpp        ! MPP library
   USE lib_fortran    ! Fortran utilities (allows no signed zero when 'key_nosignedzero' defined)  

   IMPLICIT NONE
   PRIVATE

   PUBLIC   ice_alb_init   ! called in icestp
   PUBLIC   ice_alb        ! called in iceforcing.F90 and iceupdate.F90

   REAL(wp), PUBLIC, PARAMETER ::   rn_alb_oce = 0.066   !: ocean or lead albedo (Pegau and Paulson, Ann. Glac. 2001)

   REAL(wp) , PARAMETER ::   rc1    = 0.05    ! snow thickness (only for nn_ice_alb=0)
   REAL(wp) , PARAMETER ::   rc2    = 0.10    !  "        "
   REAL(wp) , PARAMETER ::   rcloud = 0.06    ! cloud effect on albedo (only-for nn_ice_alb=0)
   REAL(wp) , PARAMETER ::   r1_c1 = 1. / rc1
   REAL(wp) , PARAMETER ::   r1_c2 = 1. / rc2
   !
   ! ** albedo namelist (namice_alb)
   INTEGER  ::   nn_ice_alb       ! type of albedo scheme: 0: Shine & Henderson-Sellers (JGR 1985)
   !                                      !                         1: "home made" based on Brandt et al. (JClim 2005)
   !                                      !                             and Grenfell & Perovich (JGR 2004)
   !                                      !                         2: Same as 1 but with melt ponds
   REAL(wp) ::   rn_alb_sdry      ! dry snow albedo
   REAL(wp) ::   rn_alb_smlt      ! melting snow albedo
   REAL(wp) ::   rn_alb_idry      ! dry ice albedo
   REAL(wp) ::   rn_alb_imlt      ! bare puddled ice albedo
   REAL(wp) ::   rn_alb_dpnd      ! ponded ice albedo

   !!----------------------------------------------------------------------
   !! NEMO/ICE 4.0 , NEMO Consortium (2017)
   !! $Id: icealb.F90 8268 2017-07-03 15:01:04Z clem $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE ice_alb( pt_ice, ph_ice, ph_snw, pafrac_pnd, ph_pnd, ld_pnd, pa_ice_cs, pa_ice_os )
      !!----------------------------------------------------------------------
      !!               ***  ROUTINE ice_alb  ***
      !!          
      !! ** Purpose :   Computation of the albedo of the snow/ice system 
      !!       
      !! ** Method  :   Two schemes are available (from namelist parameter nn_ice_alb)
      !!                  0: the scheme is that of Shine & Henderson-Sellers (JGR 1985) for clear-skies
      !!                  1: the scheme is "home made" (for cloudy skies) and based on Brandt et al. (J. Climate 2005)
      !!                                                                           and Grenfell & Perovich (JGR 2004)
      !!                  2: fractional surface-based formulation of scheme 1 (NEW)
      !!                Description of scheme 1:
      !!                  1) Albedo dependency on ice thickness follows the findings from Brandt et al (2005)
      !!                     which are an update of Allison et al. (JGR 1993) ; Brandt et al. 1999
      !!                     0-5cm  : linear function of ice thickness
      !!                     5-150cm: log    function of ice thickness
      !!                     > 150cm: constant
      !!                  2) Albedo dependency on snow thickness follows the findings from Grenfell & Perovich (2004)
      !!                     i.e. it increases as -EXP(-snw_thick/0.02) during freezing and -EXP(-snw_thick/0.03) during melting
      !!                  3) Albedo dependency on clouds is speculated from measurements of Grenfell and Perovich (2004)
      !!                     i.e. cloudy-clear albedo depend on cloudy albedo following a 2d order polynomial law
      !!                  4) The needed 4 parameters are: dry and melting snow, freezing ice and bare puddled ice
      !!
      !! ** Note    :   The parameterization from Shine & Henderson-Sellers presents several misconstructions:
      !!                  1) ice albedo when ice thick. tends to 0 is different than ocean albedo
      !!                  2) for small ice thick. covered with some snow (<3cm?), albedo is larger 
      !!                     under melting conditions than under freezing conditions
      !!                  3) the evolution of ice albedo as a function of ice thickness shows  
      !!                     3 sharp inflexion points (at 5cm, 100cm and 150cm) that look highly unrealistic
      !!
      !! References :   Shine & Henderson-Sellers 1985, JGR, 90(D1), 2243-2250.
      !!                Brandt et al. 2005, J. Climate, vol 18
      !!                Grenfell & Perovich 2004, JGR, vol 109 
      !!----------------------------------------------------------------------
      REAL(wp), INTENT(in   ), DIMENSION(:,:,:) ::   pt_ice       !  ice surface temperature (Kelvin)
      REAL(wp), INTENT(in   ), DIMENSION(:,:,:) ::   ph_ice       !  sea-ice thickness
      REAL(wp), INTENT(in   ), DIMENSION(:,:,:) ::   ph_snw       !  snow depth
      REAL(wp), INTENT(in   ), DIMENSION(:,:,:) ::   pafrac_pnd   !  melt pond relative fraction (per unit ice area)
      REAL(wp), INTENT(in   ), DIMENSION(:,:,:) ::   ph_pnd       !  melt pond depth
      LOGICAL , INTENT(in   )                   ::   ld_pnd       !  melt ponds radiatively active or not
      REAL(wp), INTENT(  out), DIMENSION(:,:,:) ::   pa_ice_cs    !  albedo of ice under clear    sky
      REAL(wp), INTENT(  out), DIMENSION(:,:,:) ::   pa_ice_os    !  albedo of ice under overcast sky
      !
      INTEGER  ::   ji, jj, jl                ! dummy loop indices
      REAL(wp) ::   zswitch, z1_c1, z1_c2     ! local scalar
      REAL(wp) ::   z1_href_pnd               !   -      -                 
      REAL(wp) ::   zalb_sm, zalb_sf, zalb_st ! albedo of snow melting, freezing, total
      !
      REAL(wp), DIMENSION(jpi,jpj,jpl) ::   zalb, zalb_it   ! intermediate variable & albedo of ice (snow free)
!! MV MP
      REAL(wp), DIMENSION(jpi,jpj,jpl) ::   zalb_pnd        ! ponded sea ice albedo
      REAL(wp), DIMENSION(jpi,jpj,jpl) ::   zalb_ice        ! bare sea ice albedo
      REAL(wp), DIMENSION(jpi,jpj,jpl) ::   zalb_snw        ! snow-covered sea ice albedo
      REAL(wp), DIMENSION(jpi,jpj,jpl) ::   zafrac_snw      ! relative snow fraction
      REAL(wp), DIMENSION(jpi,jpj,jpl) ::   zafrac_ice      ! relative ice fraction
      REAL(wp), DIMENSION(jpi,jpj,jpl) ::   zafrac_pnd      ! relative ice fraction (effective)
      !!---------------------------------------------------------------------

      !-----------------------------------------------------
      !  Snow-free albedo (no ice thickness dependence yet)
      !-----------------------------------------------------
      !
      ! Part common to nn_ice_alb = 0, 1, 2
      !
      IF ( .NOT. ld_pnd ) THEN   !--- No melt ponds OR radiatively inactive melt ponds
         ! Bare ice albedo is prescribed, with implicit assumption on pond fraction and depth
         WHERE     ( ph_snw == 0._wp .AND. pt_ice >= rt0 )   ;   zalb(:,:,:) = rn_alb_imlt
         ELSEWHERE                                           ;   zalb(:,:,:) = rn_alb_idry
         END WHERE
      ENDIF

      SELECT CASE ( nn_ice_alb )       ! select a parameterization
      !
      !              !------------------------------------------
      CASE( 0 )      !  Shine and Henderson-Sellers (1985)     !   (based on clear sky values)
         !           !------------------------------------------
         !
         !                       ! Thickness-dependent bare ice albedo
         WHERE    ( 1.5  < ph_ice                     )   ;   zalb_it = zalb
         ELSEWHERE( 1.0  < ph_ice .AND. ph_ice <= 1.5 )   ;   zalb_it = 0.472  + 2.0 * ( zalb - 0.472 ) * ( ph_ice - 1.0 )
         ELSEWHERE( 0.05 < ph_ice .AND. ph_ice <= 1.0 )   ;   zalb_it = 0.2467 + 0.7049 * ph_ice              &
            &                                                                  - 0.8608 * ph_ice * ph_ice     &
            &                                                                  + 0.3812 * ph_ice * ph_ice * ph_ice
         ELSEWHERE                                        ;   zalb_it = 0.1    + 3.6    * ph_ice
         END WHERE
         !
         IF ( ld_pnd ) THEN      ! Depth-dependent ponded ice albedo
            z1_href_pnd = 1. / 0.05    ! inverse of the characteristic length scale (Lecomte et al. 2015)
            zalb_pnd  = rn_alb_dpnd - ( rn_alb_dpnd - zalb_it ) * EXP( - ph_pnd * z1_href_pnd ) 
            !                          ! Snow-free ice albedo is a function of pond fraction
            WHERE ( ph_snw == 0._wp .AND. pt_ice >= rt0 )   
               zalb_it = zalb_it * ( 1. - pafrac_pnd  ) + zalb_pnd * pafrac_pnd 
            END WHERE
         ENDIF 
         !
!!gm: optimization ( rn_alb_smlt - rn_alb_imlt ) * r1_c2 can be computed one for all 
!!gm  before the DO-loop below
         DO jl = 1, jpl
            DO jj = 1, jpj
               DO ji = 1, jpi
                  !                    ! Freezing snow
                  ! no effect of underlying ice layer IF snow thickness > c1. Albedo does not depend on snow thick if > rc2
                  zswitch = 1._wp - MAX( 0._wp , SIGN( 1._wp , - ( ph_snw(ji,jj,jl) - rc1 ) ) )
                  zalb_sf = ( 1._wp - zswitch ) * (  zalb_it(ji,jj,jl)  &
                     &                                   + ph_snw(ji,jj,jl) * ( rn_alb_sdry - zalb_it(ji,jj,jl) ) * r1_c1  )   &
                     &    +           zswitch   * rn_alb_sdry  
                     !
                  !                    ! Melting snow
                  ! no effect of underlying ice layer. Albedo does not depend on snow thick IF > rc2
                  zswitch = MAX( 0._wp , SIGN( 1._wp , ph_snw(ji,jj,jl) - rc2 ) )
                  zalb_sm = ( 1._wp - zswitch ) * ( rn_alb_imlt + ph_snw(ji,jj,jl) * ( rn_alb_smlt - rn_alb_imlt ) * r1_c2 )   &
                     &      +         zswitch   *   rn_alb_smlt 
                     !
                  !                    ! Snow albedo
                  zswitch =  MAX( 0._wp , SIGN( 1._wp , pt_ice(ji,jj,jl) - rt0_snow ) )   
                  zalb_st =  zswitch * zalb_sm + ( 1._wp - zswitch ) * zalb_sf
                  !
                  !                    ! Surface albedo
                  zswitch             = 1._wp - MAX( 0._wp , SIGN( 1._wp , - ph_snw(ji,jj,jl) ) )
                  pa_ice_cs(ji,jj,jl) = zswitch * zalb_st + ( 1._wp - zswitch ) * zalb_it(ji,jj,jl)
                  !
               END DO
            END DO
         END DO
         !
         pa_ice_os(:,:,:) = pa_ice_cs(:,:,:) + rcloud       ! Oberhuber correction for overcast sky
         !
         !
         !           !------------------------------------------
      CASE( 1 )      !  New parameterization (2016)            !    (based on overcast sky values)
         !           !------------------------------------------
         !
         !                    compilation of values from literature (reference overcast sky values)
         !                          rn_alb_sdry = 0.85      ! dry snow
         !                          rn_alb_smlt = 0.75      ! melting snow
         !                          rn_alb_idry = 0.60      ! bare frozen ice
         !                          rn_alb_dpnd = 0.36      ! ponded-ice overcast albedo (Lecomte et al, 2015)
         !                                                  ! early melt pnds 0.27, late melt ponds 0.14 Grenfell & Perovich
         !                    Perovich et al 2002 (Sheba) => the only dataset for which all types of ice/snow were retrieved
         !                          rn_alb_sdry = 0.85      ! dry snow
         !                          rn_alb_smlt = 0.72      ! melting snow
         !                          rn_alb_idry = 0.65      ! bare frozen ice
         !                    Brandt et al 2005 (East Antarctica)
         !                          rn_alb_sdry = 0.87      ! dry snow
         !                          rn_alb_smlt = 0.82      ! melting snow
         !                          rn_alb_idry = 0.54      ! bare frozen ice
         ! 
         !              !--- Computation of snow-free ice albedo 
         z1_c1 = 1. / ( LOG(1.5) - LOG(0.05) ) 
         z1_c2 = 1. / 0.05

         !              !--- Accounting for the ice-thickness dependency
         WHERE     ( 1.5  < ph_ice                     )   ;   zalb_it = zalb
         ELSE WHERE( 0.05 < ph_ice .AND. ph_ice <= 1.5 )   ;   zalb_it = zalb + ( 0.18 - zalb ) * z1_c1 * ( LOG(1.5) - LOG(ph_ice) )
         ELSE WHERE                                        ;   zalb_it = rn_alb_oce + ( 0.18 - rn_alb_oce ) * z1_c2 * ph_ice
         END WHERE
         !
         IF ( ld_pnd ) THEN      ! Depth-dependent ponded ice albedo
            z1_href_pnd = 0.05        ! inverse of the characteristic length scale (Lecomte et al. 2015)
            zalb_pnd  = rn_alb_dpnd - ( rn_alb_dpnd - zalb_it ) * EXP( - ph_pnd * z1_href_pnd ) 
            !
            !                    ! Snow-free ice albedo is weighted mean of ponded ice and bare ice contributions
            WHERE ( ph_snw == 0._wp .AND. pt_ice >= rt0 )
               zalb_it = zalb_it * ( 1. - pafrac_pnd  ) + zalb_pnd * pafrac_pnd
            END WHERE
         ENDIF 
         !
         !              !--- Overcast sky surface albedo (accounting for snow, ice melt ponds)
         z1_c1 = 1. / 0.02
         z1_c2 = 1. / 0.03
         DO jl = 1, jpl
            DO jj = 1, jpj
               DO ji = 1, jpi
                  ! Snow depth dependence of snow albedo
                  zalb_sf = rn_alb_sdry - ( rn_alb_sdry - zalb_it(ji,jj,jl)) * EXP( - ph_snw(ji,jj,jl) * z1_c1 );
                  zalb_sm = rn_alb_smlt - ( rn_alb_smlt - zalb_it(ji,jj,jl)) * EXP( - ph_snw(ji,jj,jl) * z1_c2 );

                  ! Snow albedo (MV I guess we could use rt0 instead of rt0_snow)
                  zswitch = MAX( 0._wp , SIGN( 1._wp , pt_ice(ji,jj,jl) - rt0_snow ) )   
                  zalb_st = zswitch * zalb_sm + ( 1._wp - zswitch ) * zalb_sf

                  ! Surface albedo   
                  zswitch             = MAX( 0._wp , SIGN( 1._wp , - ph_snw(ji,jj,jl) ) )
                  pa_ice_os(ji,jj,jl) = ( 1._wp - zswitch ) * zalb_st + zswitch *  zalb_it(ji,jj,jl)

              END DO
            END DO
         END DO
         !
         !              !--- Clear sky surface albedo
         pa_ice_cs = pa_ice_os - ( - 0.1010 * pa_ice_os * pa_ice_os + 0.1933 * pa_ice_os - 0.0148 ); 
         !
         !
         !           !------------------------------------------
      CASE( 2 )      !  Fractional surface-based parameterization (2017)
         !           !------------------------------------------
         !              MV: I propose this formulation that is more elegant, and more easy to expand towards
         !              varying pond and snow fraction.
         !              Formulation 1 has issues to handle ponds and snow together that can't easily be fixed. 
         !              This one handles it better, I believe.
         !
         !                 !==  Snow, bare ice and ponded ice fractions  ==!
         !
         !                       ! Specific fractions (zafrac) refer to relative area covered by snow within each ice category
         !
         !                       !--- Effective pond fraction (for now, we prevent melt ponds and snow at the same time)
         zafrac_pnd = 0._wp
         IF ( ld_pnd ) THEN
            WHERE( ph_snw == 0._wp )   zafrac_pnd = pafrac_pnd   ! Snow fully "shades" melt ponds
         ENDIF         
         !
         !                       !--- Specific snow fraction (for now, prescribed)
         WHERE( ph_snw > 0._wp )   ;   zafrac_snw = 1.
         ELSEWHERE                 ;   zafrac_snw = 0.
         END WHERE
         !
         !                       !--- Specific ice fraction 
         zafrac_ice = 1. - zafrac_snw - zafrac_pnd
         !
         !                 !==  Snow-covered, pond-covered, and bare ice albedos  ==!
         !
         !                       !--- Bare ice albedo
         z1_c1 = 1. / ( LOG(1.5) - LOG(0.05) ) 
         z1_c2 = 1. / 0.05
         WHERE    ( 1.5  < ph_ice                     )   ;   zalb_ice = zalb
         ELSEWHERE( 0.05 < ph_ice .AND. ph_ice <= 1.5 )   ;   zalb_ice = zalb       + ( 0.18 - zalb       ) * z1_c1 *  &
            &                                                                       ( LOG(1.5) - LOG(ph_ice) )
         ELSEWHERE                                        ;   zalb_ice = rn_alb_oce + ( 0.18 - rn_alb_oce ) * z1_c2 * ph_ice
         END WHERE
         !
         z1_c1 = 1. / 0.02       !--- Snow-covered ice albedo (freezing, melting cases)
         z1_c2 = 1. / 0.03
         !
         WHERE( pt_ice < rt0_snow )   ;   zalb_snw = rn_alb_sdry - ( rn_alb_sdry - zalb_ice ) * EXP( - ph_snw * z1_c1 )
         ELSEWHERE                    ;   zalb_snw = rn_alb_smlt - ( rn_alb_smlt - zalb_ice ) * EXP( - ph_snw * z1_c2 )
         END WHERE
         !
         IF ( ld_pnd ) THEN      !--- Depth-dependent ponded ice albedo
            z1_href_pnd = 0.05        ! inverse of the characteristic length scale (Lecomte et al. 2015)
            zalb_pnd  = rn_alb_dpnd - ( rn_alb_dpnd - zalb_ice ) * EXP( - ph_pnd * z1_href_pnd ) 
         ELSE
            zalb_pnd  = rn_alb_dpnd
         ENDIF
         !                       !--- Surface albedo is weighted mean of snow, ponds and bare ice contributions
         pa_ice_os = zafrac_snw * zalb_snw  +  zafrac_pnd * zalb_pnd  +  zafrac_ice * zalb_ice
         pa_ice_cs = pa_ice_os - ( - 0.1010 * pa_ice_os * pa_ice_os + 0.1933 * pa_ice_os - 0.0148 )

      END SELECT
      !
   END SUBROUTINE ice_alb


   SUBROUTINE ice_alb_init
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE alb_init  ***
      !!
      !! ** Purpose :   initializations for the albedo parameters
      !!
      !! ** Method  :   Read the namelist namice_alb
      !!----------------------------------------------------------------------
      INTEGER  ::   ios                 ! Local integer output status for namelist read
      !!
      NAMELIST/namice_alb/ nn_ice_alb, rn_alb_sdry, rn_alb_smlt, rn_alb_idry, rn_alb_imlt, rn_alb_dpnd
      !!----------------------------------------------------------------------
      !
      REWIND( numnam_ice_ref )              ! Namelist namice_alb in reference namelist : Albedo parameters
      READ  ( numnam_ice_ref, namice_alb, IOSTAT = ios, ERR = 901)
901   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namice_alb in reference namelist', lwp )

      REWIND( numnam_ice_cfg )              ! Namelist namice_alb in configuration namelist : Albedo parameters
      READ  ( numnam_ice_cfg, namice_alb, IOSTAT = ios, ERR = 902 )
902   IF( ios /= 0 ) CALL ctl_nam ( ios , 'namice_alb in configuration namelist', lwp )
      IF(lwm) WRITE ( numoni, namice_alb )
      !
      IF(lwp) THEN                      ! Control print
         WRITE(numout,*)
         WRITE(numout,*) 'ice_alb_init: set albedo parameters'
         WRITE(numout,*) '~~~~~~~~~~~~'
         WRITE(numout,*) '   Namelist namice_alb : albedo '
         WRITE(numout,*) '      choose the albedo parameterization   nn_ice_alb  = ', nn_ice_alb
         WRITE(numout,*) '      albedo of dry snow                   rn_alb_sdry = ', rn_alb_sdry
         WRITE(numout,*) '      albedo of melting snow               rn_alb_smlt = ', rn_alb_smlt
         WRITE(numout,*) '      albedo of dry ice                    rn_alb_idry = ', rn_alb_idry
         WRITE(numout,*) '      albedo of bare puddled ice           rn_alb_imlt = ', rn_alb_imlt
         WRITE(numout,*) '      albedo of ponded ice                 rn_alb_dpnd = ', rn_alb_dpnd
      ENDIF
      !
   END SUBROUTINE ice_alb_init

#else
   !!----------------------------------------------------------------------
   !!   Default option           Dummy module      NO LIM 3.0 sea-ice model
   !!----------------------------------------------------------------------
#endif

   !!======================================================================
END MODULE icealb
